@@ -118,11 +118,12 @@ html_theme_options = {
     "body_max_width": "auto",
     "tip_bg": "lightgreen",
     "note_bg": "lightyellow",
+    "globaltoc_maxdepth": 3,
 }
 html_static_path = ["_static"]
 html_sidebars = {
     "**": [
-        "navigation.html",
+        "globaltoc.html",
         "relations.html",
         "searchbox.html",
         "mbd-sidebar-commands.html",
