@@ -1132,7 +1132,7 @@ class Reindex(_Admin, _Confirm, _Maintenance, _CatAdministrator, Call):
     anyway for whatever reason.
 
     If you just upgraded from 2.2.x, it's recommended to run it once
-    (2.4.x reindex additionally runs ``repreprop flood``, which may make
+    (2.4.x reindex additionally runs ``reprepro flood``, which may make
     some arch=all packages available for (subsequently) added
     architectures).
     """
