@@ -1,7 +1,8 @@
+import logging
+import os
+import shlex
 import subprocess
 import time
-import os
-import logging
 
 from mini_buildd import config, util
 
@@ -46,7 +47,7 @@ class Call():
         command line more human-readable and "likely-suitable"
         for cut and paste to a shell.
         """
-        return util.PyCompat.shlex_join(call)
+        return shlex.join(call)
 
     def __init__(self, call, run_as_root=False, **kwargs):
         self.call = ["sudo", "-n"] + call if run_as_root else call
@@ -65,7 +66,7 @@ class Call():
         # Convenience 'label' for log output
         self.label = f"{'#' if run_as_root else '?'} {call[0]}.."
 
-        LOG.debug(f"Called with returncode {self.result.returncode}: {self._call2shell(self.call)}")
+        LOG.debug("Called with returncode %s: %s", self.result.returncode, self._call2shell(self.call))
 
     @classmethod
     def _bos2str(cls, value, errors="replace"):
@@ -100,7 +101,7 @@ class Call():
         level = level if level is not None else logging.DEBUG if self.result.returncode == 0 else logging.ERROR
         for prefix, output in [("stdout", self.stdout), ("stderr", self.stderr)]:
             for line in output.splitlines():
-                LOG.log(level, f"{self.label} ({prefix}): {line}")
+                LOG.log(level, "%s (%s): %s", self.label, prefix, line)
         return self
 
     def success(self):
@@ -126,7 +127,7 @@ def call_sequence(calls, run_as_root=False, rollback_only=False, **kwargs):
             if calls[i][1]:
                 Call(calls[i][1], run_as_root=run_as_root, **kwargs).log()
             else:
-                LOG.debug(f"Skipping empty rollback call sequent {i}")
+                LOG.debug("Skipping empty rollback call sequent %s", i)
 
     if rollback_only:
         rollback(len(calls) - 1)
@@ -137,10 +138,10 @@ def call_sequence(calls, run_as_root=False, rollback_only=False, **kwargs):
                 if c[0]:
                     Call(c[0], run_as_root=run_as_root, **kwargs).check()
                 else:
-                    LOG.debug(f"Skipping empty call sequent {i}")
+                    LOG.debug("Skipping empty call sequent %s", i)
                 i += 1
         except BaseException:
-            LOG.error(f"Sequence failed at: {i} (rolling back)")
+            LOG.error("Sequence failed at: %s (rolling back)", i)
             rollback(i)
             raise
 
@@ -164,7 +165,7 @@ def call_with_retry(call, retry_max_tries=5, retry_sleep=1, retry_failed_cleanup
         except BaseException as e:
             if t >= retry_max_tries - 1:
                 raise
-            LOG.warning(f"Retrying call in {retry_sleep} seconds [retry #{t}]: {e}")
+            LOG.warning("Retrying call in %s seconds [retry #{t}]: %s", retry_sleep, e)
             if retry_failed_cleanup is not None:
                 retry_failed_cleanup()
             time.sleep(retry_sleep)
