@@ -1,31 +1,26 @@
-import os
-import time
-import shutil
-import multiprocessing
-import datetime
-import tempfile
-import hashlib
-import json
-import re
-import logging
 import contextlib
+import datetime
+import hashlib
 import http
-import shlex
+import json
+import logging
+import multiprocessing
+import os
+import platform
+import re
+import shutil
+import tempfile
+import time
 import traceback
-import ssl
-
-import packaging.version
 
 import dateparser
-
+import debian.debian_support
 import django
 import django.core.exceptions
-
+import packaging.version
 import twisted
 
-import debian.debian_support
-
-from mini_buildd import config, version, net
+from mini_buildd import config, net, version
 
 LOG = logging.getLogger(__name__)
 
@@ -38,6 +33,7 @@ class Versions(dict):
 
     def __init__(self):
         self.version = __version__
+        self["python"] = platform.python_version()
         self["django"] = django.get_version()
         self["twisted"] = twisted.version.public()
 
@@ -55,17 +51,6 @@ def http_endpoint(number=0):
     return net.ServerEndpoint(config.HTTP_ENDPOINTS[number], protocol="http")
 
 
-def daemon():
-    """
-    Import ``daemon`` module and return ``Daemon`` singleton (sort-of dependency injection)
-
-    Use this where you need the ``models`` module but can't do a 'proper import' -- as django needs to be
-    configured first.
-    """
-    from mini_buildd.daemon import Daemon
-    return Daemon()
-
-
 def models():
     """
     Import and return ``models`` module (sort-of dependency injection)
@@ -73,7 +58,7 @@ def models():
     Use this where you need the ``models`` module but can't do a 'proper import' -- as django needs to be
     configured first.
     """
-    from mini_buildd import models as models_module
+    from mini_buildd import models as models_module  # pylint: disable=import-outside-toplevel
     return models_module
 
 
@@ -92,40 +77,22 @@ def fopen(path, mode="r", **kwargs):
     return open(path, mode, encoding=config.CHAR_ENCODING, **kwargs)
 
 
-class PyCompat:
-    """Misc helpers to stay compatible with python <= 3.6 (see ``debian/control``)"""
-
-    @staticmethod
-    def removeprefix(s, prefix):
-        """
-        .. attention:: **compat** (``python < 3.9``): ``str.removeprefix``
-        """
-        if s.startswith(prefix):
-            return s[len(prefix):]
-        return s
-
-    @staticmethod
-    def shlex_join(split_command):
-        """
-        .. attention:: **compat** (``python < 3.8``): ``shlex.join``
-        """
-        return " ".join(shlex.quote(arg) for arg in split_command)
-
-
 class Rfc7807:
     def __init__(self, status, detail=None):
         self.status = status if isinstance(status, http.HTTPStatus) else http.HTTPStatus(status)
-        self.detail = f"No public details (see <a href='{config.URIS['log']['view'].uri}daemon.log'>daemon.log</a>)" if detail is None else detail
+        self.detail = "No public details (see service log)" if detail is None else detail
 
     def __str__(self):
         return f"{self.detail} (HTTP {self.status.value} {self.status.description})"
 
     def to_json(self):
-        return {"type": "about:blank",  # No special semantics. Status code is HTTP status code, title is HTTP status message
-                "status": self.status.value,
-                "title": self.status.description,
-                "detail": self.detail,
-                "instance": "about:blank"}
+        return {
+            "type": "about:blank",  # No special semantics. Status code is HTTP status code, title is HTTP status message
+            "status": self.status.value,
+            "title": self.status.description,
+            "detail": self.detail,
+            "instance": "about:blank",
+        }
 
     @classmethod
     def from_json(cls, data):
@@ -178,8 +145,19 @@ class HTTPInternal(HTTPError):
         super().__init__(http.HTTPStatus.INTERNAL_SERVER_ERROR, detail)
 
 
+def raised_info(exception):
+    """Try to get filename:lineno from last raise (i.e., to log even in 'production mode'/exc_info=False)"""
+    try:
+        tb = exception.__traceback__
+        while tb.tb_next is not None:
+            tb = tb.tb_next
+        return f"{os.path.basename(tb.tb_frame.f_code.co_filename)}:{tb.tb_lineno}"
+    except:  # noqa (pep8 E722)  # pylint: disable=bare-except
+        return ""
+
+
 def log_exception(log, message, exception, level=logging.WARNING):
-    log.log(level, f"{message}: {exception}", exc_info=log.isEnabledFor(logging.DEBUG))
+    log.log(level, "%s: %s [raised in: %s]", message, exception, raised_info(exception), exc_info=log.isEnabledFor(logging.DEBUG))
 
 
 def log_stack(log, level=logging.INFO):
@@ -207,30 +185,6 @@ def check_program(path, deb=None):
         raise HTTPUnavailable(f"Program not found: {path} (please install Debian package {deb})")
 
 
-def systemcert_workaround():
-    """
-    Set environment ``SSL_CERT_FILE`` to point to system's certificate store (ca-certificates) when not using ``libssl3``
-
-    .. note:: **compat** (``< libssl3``): ``python`` fails for (self-signed) certificate even though it's enrolled with system's ``ca-certificates``?
-
-        You will get some error like ``certificate verify failed: self signed certificate``.
-
-        This seems to occur with ``libssl1``, while it seems fine with ``libssl3`` (even though :debbug:`805646` never got closed).
-
-        Check the package deps on ``libpythonX.Y-minimal`` to see if it was build against ``libssl1`` or ``libssl3`` (or mini-buildd's log for warning message).
-
-        Simplest workaround is to set ``SSL_CERT_FILE`` environment, for example like so:
-
-        .. code:: bash
-
-          export SSL_CERT_FILE="/etc/ssl/certs/ca-certificates.crt"
-
-    """
-    if ssl.OPENSSL_VERSION_INFO[0] < 3:
-        os.environ["SSL_CERT_FILE"] = "/etc/ssl/certs/ca-certificates.crt"
-        LOG.warning("Setting environment SSL_CERT_FILE='/etc/ssl/certs/ca-certificates.crt' for openssl < 3 (use system's cert store)")
-
-
 def singularize(s):
     """Singularize some english nouns from plural"""
     if s:
@@ -337,7 +291,7 @@ class Datetime:
             raise HTTPBadRequest(f"Date parse failed on: '{date_string}' ({Datetime.parse.__doc__})")
         if Datetime.is_naive(dt):
             dt = dt.astimezone()
-            LOG.debug(f"Naive timestamp converted: {date_string} => {dt}")
+            LOG.debug("Naive timestamp converted: %s => %s", date_string, dt)
         return dt
 
 
@@ -401,19 +355,6 @@ class StopWatch():
                 "delta": self.delta().total_seconds()}
 
 
-class Singleton(type):
-    __INSTANCES = {}
-
-    def __call__(cls, *args, **kwargs):  # noqa (pep8 N805)
-        if cls not in cls.__INSTANCES:
-            cls.__INSTANCES[cls] = super(Singleton, cls).__call__(*args, **kwargs)
-        return cls.__INSTANCES[cls]
-
-    def destroy(cls):  # noqa (pep8 N805)
-        if cls in cls.__INSTANCES:
-            del cls.__INSTANCES[cls]
-
-
 class TmpDir():
     """Temporary dir class (use with ``with contextlib.closing()...``, or as mixin class)"""
 
@@ -423,9 +364,12 @@ class TmpDir():
 
     def close(self):
         if "keep" in config.DEBUG:
-            LOG.warning(f"DEBUG MODE('keep'): Skipping removal of temporary dir '{self.tmpdir}'")
+            LOG.warning("DEBUG MODE('keep'): Skipping removal of temporary dir '%s'", self.tmpdir)
         else:
-            shutil.rmtree(self.tmpdir, onerror=lambda function, path, excinfo: LOG.warning(f"Could not remove temporary '{path}': {excinfo[1]}"))
+            if VERSIONS.has("python", "3.12"):
+                shutil.rmtree(self.tmpdir, onexc=lambda function, path, excinfo: LOG.warning("Could not remove temporary '%s': %s", path, excinfo))  # pylint: disable=unexpected-keyword-arg
+            else:
+                shutil.rmtree(self.tmpdir, onerror=lambda function, path, excinfo: LOG.warning("Could not remove temporary '%s': %s", path, excinfo[1]))  # pylint: disable=deprecated-argument
 
 
 @contextlib.contextmanager
@@ -463,7 +407,7 @@ def attempt(func, *args, retval_on_failure=None, **kwargs):
 def measure(func, *args, **kwargs):
     start = time.process_time()
     result = func(*args, **kwargs)
-    LOG.debug(f"{func}: {time.process_time() - start} seconds")
+    LOG.debug("%s: %s seconds", func, time.process_time() - start)
     return result
 
 
@@ -545,7 +489,7 @@ def rmdirs(path):
     """Remove path recursively -- succeed even if it does not exist in the first place"""
     if os.path.exists(path):
         shutil.rmtree(path)
-        LOG.info(f"Directory removed recursively: {path}")
+        LOG.info("Directory removed recursively: %s", path)
 
 
 def json_pretty(json_obj):
